---------------------------------------------------------
-- Keep track of built objects
function builtEntity(event)
	local ent = event.created_entity
	local name = ent.name
	if name == "chunk-marker" then
		markChunk(ent.surface, ent.position, "hazard-concrete-left")
		ent.destroy()
	elseif name == "chunk-unmarker" then
		unmarkChunk(ent.surface, ent.position, "hazard-concrete-left")
		ent.destroy()
	end
end
script.on_event(defines.events.on_built_entity, builtEntity)
script.on_event(defines.events.on_robot_built_entity, builtEntity)

---------------------------------------------------------
-- Return the chunk position from a tile position
-- Not sure if this is the actual chunk, but it should give nice 32x32 blocks
function getChunkFromPosition(position)
	return {x = math.floor(position.x / 32) * 32, y = math.floor(position.y / 32) * 32}
end

---------------------------------------------------------
-- Return a table of border positions of a chunk (top left corner)
-- Not sure if this is the actual chunk, but it should give nice 32x32 blocks
function getChunkBoundary(position)
	local pos = position
	local boundary_positions = {}
	for i = 0, 31 do
		-- North/south edges
		table.insert(boundary_positions, {x = pos.x + i, y = pos.y})
		table.insert(boundary_positions, {x = pos.x + i, y = pos.y + 31})
		-- West/east edges
		table.insert(boundary_positions, {x = pos.x, y = pos.y + i})
		table.insert(boundary_positions, {x = pos.x + 31, y = pos.y + i})
	end
	return boundary_positions
end

---------------------------------------------------------
-- Create a border around a given chunk
function markChunk(surface, position, material)
	local chunkTopLeft = getChunkFromPosition(position)
	local boundary_positions = getChunkBoundary(chunkTopLeft)
	
	local boundary = {}
	for __, pos in pairs(boundary_positions) do
		local tile = surface.get_tile(pos.x, pos.y)
		if not isWater(tile) then
			table.insert(boundary, {name = material, position = pos})
		end
	end
	
	surface.set_tiles(boundary)
end

---------------------------------------------------------
-- Delete a border around a given chunk
function unmarkChunk(surface, position, material)
	local chunkTopLeft = getChunkFromPosition(position)
	local boundary_positions = getChunkBoundary(chunkTopLeft)
	
	local boundary = {}
	for __, pos in pairs(boundary_positions) do
		local tile = surface.get_tile(pos.x, pos.y)
		if tile.name == material then
			table.insert(boundary, {name = "grass", position = pos})
		end
	end
	
	surface.set_tiles(boundary)
end

---------------------------------------------------------
-- Returns whether the tile is a water tile
function isWater(tile)
	local name = tile.name
	return name == "water" or name == "water-green" or name == "deepwater" or name == "deepwater-green"
end

---------------------------------------------------------
-- Utility function to print to all players
function pall(str)
	if game and game.players then
		for __, player in pairs(game.players) do
			player.print(str)
		end
	end
end